import sys

import openai
from gpt.util import get_token_amount_from_string


def openai_request(prompt: str, model: str, temperature: float):
    TOKEN_LIMIT = 6500
    if get_token_amount_from_string(prompt) > TOKEN_LIMIT:
        print(f"Token amount of {get_token_amount_from_string(prompt)} is to big. Stay below {TOKEN_LIMIT}!")
        return
    response = openai.ChatCompletion.create(
        model=model,
        temperature=temperature,
        stream=True,
        messages=[
            {"role": "system",
             "content": "Help answer user questions, provide solutions step by step."},
            {"role": "user", "content": prompt}
        ]
    )

    return response


def print_streamed_response(response):
    # create variables to collect the stream of events
    collected_events = []
    # iterate through the stream of events
    for event in response:
        collected_events.append(event)  # save the event response
        if event['choices'][0]['delta'].get('content') is not None:
            event_text = event['choices'][0]['delta']['content']  # extract the text
            sys.stdout.write(event_text)
            sys.stdout.flush()  # ensures output is displayed immediately



import tiktoken


def get_token_amount_from_string(text: str):
    # cl100k_base is for gpt-4 and gpt-3.5-turbo
    encoding = tiktoken.get_encoding("cl100k_base")
    return len(encoding.encode(text))


import openai
from arguments import get_args, get_multiline_input
from config.credentials import get_api_key
from gpt.request import openai_request, print_streamed_response


def main():
    openai.api_key = get_api_key()

    args = get_args()

    model = args.m
    temperature = args.t
    verbose = args.v
    input_string = args.p

    if input_string is None:
        input_string = get_multiline_input()

    if verbose:
        print(f"Selected model: {model}")
        print(f"Selected temperature: {temperature}")
        print(f"Prompt: {input_string}")
        print("Response:")

    response = openai_request(input_string, model, temperature)
    print_streamed_response(response)


if __name__ == "__main__":
    main()
